import React from 'react';
import ImmutablePropTypes from 'react-immutable-proptypes';
import PropTypes from 'prop-types';
import { is } from 'immutable';
import IconButton from './icon_button';
import { defineMessages, injectIntl, FormattedMessage } from 'react-intl';
import { isIOS } from '../is_mobile';
import { autoPlayGif, displaySensitiveMedia } from '../initial_state';

const messages = defineMessages({
  toggle_visible: { id: 'media_gallery.toggle_visible', defaultMessage: 'Toggle visibility' },
  sensitive: { id: 'media_gallery.sensitive', defaultMessage: 'Sensitive' },
});

class Item extends React.PureComponent {

  static propTypes = {
    attachment: ImmutablePropTypes.map.isRequired,
    index: PropTypes.number.isRequired,
    onClick: PropTypes.func.isRequired,
    displayWidth: PropTypes.number,
  };

  static defaultProps = {
    index: 0,
  };

  handleMouseEnter = (e) => {
    if (this.hoverToPlay()) {
      e.target.play();
    }
  }

  handleMouseLeave = (e) => {
    if (this.hoverToPlay()) {
      e.target.pause();
      e.target.currentTime = 0;
    }
  }

  hoverToPlay () {
    const { attachment } = this.props;
    return !autoPlayGif && attachment.get('type') === 'gifv';
  }

  handleClick = (e) => {
    const { index, onClick } = this.props;

    if (e.button === 0) {
      e.preventDefault();
      onClick(index);
    }

    e.stopPropagation();
  }

  render () {
    const { attachment } = this.props;

    let thumbnail = '';

    if (attachment.get('type') === 'image') {
      const focusX = attachment.getIn(['meta', 'focus', 'x']) || 0;
      const focusY = attachment.getIn(['meta', 'focus', 'y']) || 0;
      const x      = ((focusX /  2) + .5) * 100;
      const y      = ((focusY / -2) + .5) * 100;

      thumbnail = (
        <a
          className="attachement"
          href={attachment.get('remote_url') || attachment.get('url')}
          onClick={this.handleClick}
          target='_blank'
        >
          <img
            src={attachment.get('url')}
            alt={attachment.get('description')}
            title={attachment.get('description')}
            style={{ objectPosition: `${x}% ${y}%` }}
          />
        </a>
      );
    } else if (attachment.get('type') === 'audio') {
      thumbnail = (
        <audio
          className="attachement"
          aria-label={attachment.get('description')}
          role='application'
          src={attachment.get('url')}
          loop
          controls
        />
      );
    } else if (attachment.get('type') === 'gifv') {
      const autoPlay = !isIOS() && autoPlayGif;

      thumbnail = (
        <video
          className="attachement"
          aria-label={attachment.get('description')}
          role='application'
          src={attachment.get('url')}
          onClick={this.handleClick}
          onMouseEnter={this.handleMouseEnter}
          onMouseLeave={this.handleMouseLeave}
          autoPlay={autoPlay}
          loop
          muted
        />
      );
    }

    return thumbnail;
  }

}

@injectIntl
export default class MediaGallery extends React.PureComponent {

  static propTypes = {
    sensitive: PropTypes.bool,
    media: ImmutablePropTypes.list.isRequired,
    onOpenMedia: PropTypes.func.isRequired,
    intl: PropTypes.object.isRequired,
  };

  state = {
    visible: !this.props.sensitive || displaySensitiveMedia,
  };

  componentWillReceiveProps (nextProps) {
    if (!is(nextProps.media, this.props.media)) {
      this.setState({ visible: !nextProps.sensitive });
    }
  }

  handleOpen = () => {
    this.setState({ visible: !this.state.visible });
  }

  handleClick = (index) => {
    this.props.onOpenMedia(this.props.media, index);
  }

  handleRef = (node) => {
    if (node /*&& this.isStandaloneEligible()*/) {
      // offsetWidth triggers a layout, so only calculate when we need to
      this.setState({
        width: node.offsetWidth,
      });
    }
  }

  isStandaloneEligible() {
    const { media, standalone } = this.props;
    return standalone && media.size === 1 && media.getIn([0, 'meta', 'small', 'aspect']);
  }

  render () {
    const {
      handleClick,
      handleOpen,
    } = this;
    const {
      intl,
      media,
      sensitive,
    } = this.props;
    const { visible } = this.state;

    let render_sensitiveInfo = (
      <button
        className="icon-button overlayed sensitive-info"
        onClick={handleOpen}
        title={intl.formatMessage(messages.toggle_visible)}
      >
        👁
        {sensitive ? (
          <span className='sensitive-marker'>
            <FormattedMessage id='status.sensitive' defaultMessage='Sensitive' />
          </span>
        ) : null}
      </button>
    );

    let warning;

    if (sensitive) {
      warning = <FormattedMessage id='status.sensitive_warning' defaultMessage='Sensitive content' />;
    } else {
      warning = <FormattedMessage id='status.media_hidden' defaultMessage='Media hidden' />;
    }

    let render_mediaSpoiler = (
      <button
        className='media-spoiler'
        onClick={handleOpen}
      >
        <span className='media-spoiler__warning'>
          {warning}
        </span>
        <span className='media-spoiler__trigger'>
          <FormattedMessage id='status.sensitive_toggle' defaultMessage='Click to view' />
        </span>
      </button>
    );

    let render_attachement = ( 'media_error' );

    switch (media.lenght) {
    case 0: break;
    case 1: render_attachement = (
      <Item
        attachment={media.get(0)}
        onClick={handleClick}
      />
    );
      break;
    default: render_attachement = (
      media.map(
        (attachment, i) => (
          <Item
            attachment={attachment}
            index={i}
            key={attachment.get('id')}
            onClick={handleClick}
          />
        )
      )
    );

    };

    if (visible) {
      return (
        <div className='media-gallery'>
          { render_sensitiveInfo } { render_attachement }
        </div>
      );
    } else {
      return (
        <div className='media-gallery'>
          { render_mediaSpoiler }
        </div>
      );
    }
  }

}
